package isolationslevel;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */
import java.sql.*;

public class IsolationsLevel {
  public static void main(String args[]) {
    System.out.println("Abfrage der Kaffe Datenbank mit einem Join");
    String url = "jdbc:odbc:coffeebreak";
    Connection con;
    // ohne distinct werden alle Records zweimal angezeigt
    String query = "select distinct SUPPLIERS.SUP_NAME, COFFEES.COF_NAME " +
                    "from COFFEES, SUPPLIERS " +
                    "where SUPPLIERS.SUP_NAME like 'Acme, Inc.' "+
                    "and SUPPLIERS.SUP_ID = COFFEES.SUP_ID";
    Statement stmt;

    try {
      System.out.println("Treiber laden : ODBC JDBC");
      Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
    } catch(java.lang.ClassNotFoundException e) {
      System.err.print("Treiberfehler : ClassNotFoundException: ");
      System.err.println(e.getMessage());
    }

    try {
      System.out.println("Verbindung aufbauen");
      con = DriverManager.getConnection (url, "", "");
      System.out.println("Verbindung hergestellt");
      boolean bAC = con.getAutoCommit();
      System.out.println("AutoCommit Status ermittelt");
      int iIL = con.getTransactionIsolation();
      boolean bRO = con.isReadOnly();
      System.out.println("System-Informationen");
      System.out.println("AutoCommit : "+bAC);
      System.out.println("TransactionIsolation : "+iIL);
      //
      if(iIL == Connection.TRANSACTION_NONE) {
        System.out.println("TRANSACTION_NONE");
        System.out.println("Transaktionen werden nicht unterstuetzt");
      }
      if(iIL == Connection.TRANSACTION_READ_COMMITTED) {
        System.out.println("TRANSACTION_READ_COMMITTED");
        System.out.println("Dirty Reads werden abgefangen; non-repeatable phantom reads koennen auftreten.");
        System.out.println("Dieser Level verhindert, dass Daten einer unbestaetigten Transaktion gelesen werden.");
      }
      if(iIL == Connection.TRANSACTION_READ_UNCOMMITTED) {System.out.println("TRANSACTION_READ_UNCOMMITTED");
        System.out.println("Dirty Reads, non-repeatable reads und phantom reads knnen auftreten.");
        System.out.println("Dieser Level gestattet es einer Transaktionen Daten vor der Bestaetigung der Transaktion zu lesen: dirty reads.");
        System.out.println("Falls die Transaktion rueckgaengig gemacht wird, stehen ungueltige Daten zur Verfuegung.");
      }
      if(iIL == Connection.TRANSACTION_REPEATABLE_READ) {System.out.println("TRANSACTION_REPEATABLE_READ");
        System.out.println("Dirty reads and non-repeatable reads are prevented; phantom reads can occur. This level prohibits a transaction from reading a row with uncommitted changes in it, and it also prohibits the situation where one transaction reads a row, a second transaction alters the row, and the first transaction rereads the row, getting different values the second time (a 'non-repeatable read')");
      }
      if(iIL == Connection.TRANSACTION_SERIALIZABLE) {System.out.println("TRANSACTION_SERIALIZABLE");
        System.out.println("Dirty reads, non-repeatable reads and phantom reads are prevented. This level includes the prohibitions in TRANSACTION_REPEATABLE_READ and further prohibits the situation where one transaction reads all rows that satisfy a WHERE condition, a second transaction inserts a row that satisfies that WHERE condition, and the first transaction rereads for the same condition, retrieving the additional 'phantom' row in the second read");
      }
      System.out.println("isReadOnly : "+bRO);
      System.out.println("Setzen von AutoCommit auf FALSE");
      con.setAutoCommit(false);
      System.out.println("Setzen von ReadOnly auf TRUE");
      con.setReadOnly(true);
      bAC = con.getAutoCommit();
      iIL = con.getTransactionIsolation();
      bRO = con.isReadOnly();
      System.out.println("System-Informationen");
      System.out.println("AutoCommit : "+bAC);
      System.out.println("TransactionIsolation : "+iIL);
      System.out.println("isReadOnly : "+bRO);

      System.out.println("Abfrage vorbereiten");
      stmt = con.createStatement();

      System.out.println("Resultset aufbauen");
      ResultSet rs = stmt.executeQuery(query);

      System.out.println("Und hier das Ergebnis:");
      System.out.println("    " + "Supplier" + ", " + "Coffee:");
      while (rs.next()) {
        String supName = rs.getString(1);
        String cofName = rs.getString(2);
        System.out.println("    " + supName + ", " + cofName);
      }

      System.out.println("Das war's");

      con.commit();
      // vor dem Close muss man die Transaktion abschliessen, sonst tritt ein SQL Error auf.
      stmt.close();
      con.close();

    } catch(SQLException ex) {
      System.err.print("SQLException: ");
      System.err.println(ex.getMessage());
    }
  }
}

