package metadaten;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */

import java.sql.*;

public class MetaDaten {
  public static void main(String args[]) {
    System.out.println("Abfrage der Kaffe Datenbank mit einem Join");
    String url = "jdbc:odbc:coffeebreak";
    Connection con;
    // ohne distinct werden alle Records zweimal angezeigt
    String query = "select distinct SUPPLIERS.SUP_NAME, COFFEES.COF_NAME " +
                    "from COFFEES, SUPPLIERS " +
                    "where SUPPLIERS.SUP_NAME like 'Acme, Inc.' "+
                    "and SUPPLIERS.SUP_ID = COFFEES.SUP_ID";
    Statement stmt;
    try {
      System.out.println("Treiber laden : ODBC JDBC");
      Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
    } catch(java.lang.ClassNotFoundException e) {
      System.err.print("Treiberfehler : ClassNotFoundException: ");
      System.err.println(e.getMessage());
    }

    try {
      System.out.println("Verbindung aufbauen");
      con = DriverManager.getConnection (url, "", "");
      boolean bAC = con.getAutoCommit();
      int iIL = con.getTransactionIsolation();
      boolean bRO = con.isReadOnly();
      System.out.println("System-Informationen");
      System.out.println("AutoCommit : "+bAC);
      System.out.println("TransactionIsolation : "+iIL);
      System.out.println("isReadOnly : "+bRO);
      System.out.println("Setzen von AutoCommit auf FALSE");
      con.setAutoCommit(false);
      System.out.println("Setzen von ReadOnly auf TRUE");
      con.setReadOnly(true);
      bAC = con.getAutoCommit();
      iIL = con.getTransactionIsolation();
      bRO = con.isReadOnly();
      System.out.println("System-Informationen");
      System.out.println("AutoCommit : "+bAC);
      System.out.println("TransactionIsolation : "+iIL);
      System.out.println("isReadOnly : "+bRO);
      System.out.println("Abfrage vorbereiten");
      stmt = con.createStatement();
      System.out.println("Resultset aufbauen");
      ResultSet rs = stmt.executeQuery(query);
      System.out.println("Bestimmung der MetaDaten des Result Sets");
      ResultSetMetaData rsmd;
      rsmd = rs.getMetaData();
      int iRSCount = rsmd.getColumnCount();
      System.out.println("Anzahl Spalten im ResultSet : "+iRSCount);
      // jetzt fragen wir einfach einige Grssen ab
      for (int i = 1; i <= iRSCount; i++) {
        String colName = rsmd.getColumnName(i);
        String tableName = rsmd.getTableName(i);
        String name = rsmd.getColumnTypeName(i);
        boolean caseSen = rsmd.isCaseSensitive(i);
        boolean writable = rsmd.isWritable(i);
        System.out.println("Informationen fr die Spalte " + colName);
        System.out.println("    Die Spalte gehrt zur Tabelle " + tableName+"<<ResultSet MetaDaten>>");
        System.out.println("    DBMS ist vom Typ " + name);
        System.out.println("    Gross-Kleinschreibung:  " + caseSen);
        System.out.println("    Schreibbar:  " + writable);
        System.out.println("");
      }
      System.out.println("Und hier das Ergebnis:");
      System.out.println("\t\tSupplier\tCoffee:");
      while (rs.next()) {
        String supName = rs.getString(1);
        String cofName = rs.getString(2);
        System.out.println("\t\t" + supName + "\t" + cofName);
      }
      System.out.println("Das war's");
      stmt.close();
      con.close();
    } catch(SQLException ex) {
      System.err.print("SQLException: ");
      System.err.println(ex.getMessage());
    }
  }
}

