package scrollableresultsets;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */

import java.sql.*;

public class ScrollableResultSets  {
  public static void main(String args[]) {
    System.out.println("Abfrage einer ODBC Datenbank mit Hilfe von SQL");
    String url = "jdbc:odbc:coffeebreak";
    Connection con;
    String query = "select SUPPLIERS.SUP_NAME, COFFEES.COF_NAME " +
                    "from COFFEES, SUPPLIERS " +
                    "where SUPPLIERS.SUP_NAME like 'Acme, Inc.' and " +
                      "SUPPLIERS.SUP_ID = COFFEES.SUP_ID";
    Statement stmt;
    try {
      System.out.println("JDBC ODBC Driver");
      Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
    } catch(java.lang.ClassNotFoundException e) {
      System.err.print("ClassNotFoundException: ");
      System.err.println(e.getMessage());
    }
    try {
      System.out.println("Verbindungsaufbau mit der Datenbank");
      con = DriverManager.getConnection(url,"", "");
      System.out.println("Uebergabe der Abfrage");
      stmt = con.createStatement( ResultSet.TYPE_SCROLL_SENSITIVE,
                                  ResultSet.CONCUR_UPDATABLE);
      System.out.println("Fetch : 25 Datensaetze");
      stmt.setFetchSize(25);
      ResultSet rs = stmt.executeQuery(query);
      ResultSetMetaData rsmd = rs.getMetaData();
      int numberOfColumns = rsmd.getColumnCount();
      int rowCount = 1;
      while (rs.next()) {
        System.out.println("Row " + rowCount + ":  ");
        for (int i = 1; i <= numberOfColumns; i++) {
          System.out.print("   Column " + i + ":  ");
          System.out.println(rs.getString(i));
        }
        System.out.println("");
        rowCount++;
      }
      /**
       * Scrollable ResultSets
       * Scrollen vorwrts
       */
      System.out.println("\nScrollen vorwaerts");
      try {
        Statement stmt1 = con.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE,
                                     ResultSet.CONCUR_READ_ONLY);

        ResultSet srs1 = stmt1.executeQuery("SELECT COF_NAME, PRICE FROM COFFEES");
        String[ ] tabs = { "\t\t", "\t", "\t\t", "\t\t", "\t"};
        int iTab = 0;
        while (srs1.next()) {
	  String name = srs1.getString("COF_NAME");
	  float price = srs1.getFloat("PRICE");
	  System.out.println(name + tabs[iTab] + price);
          iTab++;
        }
        /**
         * Scrollable ResultSet
         * Scrollen rckwrts
         */
        System.out.println("\nScrollen rueckwaerts");
        Statement stmt2 = con.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,
                                     ResultSet.CONCUR_READ_ONLY);
        ResultSet srs2 = stmt2.executeQuery("SELECT COF_NAME, PRICE FROM COFFEES");
        srs2.afterLast();
        String tabs2[] = { "\t", "\t", "\t\t", "\t\t", "\t\t"};
        iTab = 0;
        while (srs2.previous()) {
	  String name = srs2.getString("COF_NAME");
	  float price = srs2.getFloat("PRICE");
	  System.out.println(name + tabs2[iTab] + price);
          iTab++;
        }
        /**
         * Scrollable ResultSet
         * absolute Position
         */
        System.out.println("\nPositionieren");
        Statement stmt3 = con.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,
                                     ResultSet.CONCUR_READ_ONLY);
        ResultSet srs3 = stmt3.executeQuery("SELECT COF_NAME, PRICE FROM COFFEES");
        String[ ] tabs3 = { "\t\t", "\t", "\t\t", "\t\t", "\t"};
        iTab = 0;
        srs3.absolute(4);
        System.out.println("Absolute Position : 4");
        int rowNum = srs3.getRow(); // rowNum = 4
        String name = srs3.getString("COF_NAME");
        float price = srs3.getFloat("PRICE");
        System.out.println("Zeile 4 : "+name + tabs3[3] + price);

        try {
        System.out.println("beforeFirst: -3 = 1");
        //srs3.beforeFirst(); // Fehler
        //srs3.relative(-3);  // Fehler
        srs3.relative(-2);  // geht
        rowNum = srs3.getRow(); // rowNum = 1+1
        name = srs3.getString("COF_NAME");
        price = srs3.getFloat("PRICE");
        System.out.println("Zeile 1 : "+name + tabs3[0] + price);
        } catch(SQLException se) { System.out.println("Geht nicht"); }

        try {
        System.out.println("Relative Position : 2 = 3");
        srs3.relative(2);
        rowNum = srs3.getRow(); // rowNum = 3
        name = srs3.getString("COF_NAME");
        price = srs3.getFloat("PRICE");
        System.out.println("Zeile 3 : "+name + tabs3[2] + price);
        } catch(SQLException se) { System.out.println("Geht nicht"); }

      } catch(SQLException se) {se.printStackTrace(); }
      /**
       * Mutieren eines Scrollable ResultSets
       */
       System.out.println("\nUpdatable ResultSet");
      Statement stmt4 = con.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE,
				     ResultSet.CONCUR_UPDATABLE);
      ResultSet uprs = stmt4.executeQuery("SELECT COF_NAME, PRICE FROM COFFEES");
      String[ ] tabs = { "\t\t", "\t", "\t\t", "\t\t", "\t"};
      int iTab = 0;
      while (uprs.next()) {
	String name = uprs.getString("COF_NAME");
	float price = uprs.getFloat("PRICE");
	System.out.println(name + tabs[iTab] + price);
        iTab++;
      }
      // Access : liest und schliesst: ResultSet is closed
      uprs.close();
      System.out.println("\nUpdates");
      ResultSet uprs1 = stmt4.executeQuery("SELECT COF_NAME, PRICE FROM COFFEES");
      // nchste Anweisung fhrt bei Access zu Fehler
      /*stmt4.executeUpdate("UPDATE COFFEES SET PRICE = 10.99 WHERE COF_NAME = 'FRENCH_ROAST_DECAF'");
      uprs1.last();
      uprs1.updateFloat("PRICE", 19.99f);
      uprs1.updateRow();
      */

      iTab = 0;
      while (uprs1.next()) {
	String name = uprs1.getString("COF_NAME");
	float price = uprs1.getFloat("PRICE");
	System.out.println(name + tabs[iTab] + price);
        iTab++;
      }

      /**
       * Metadata
       */
      boolean meta = false;
      if (meta) {
       try {
        Thread.sleep(500);}
      catch (Exception e) {}
      System.out.println("METADATA");
      int rsmdColuumnCount = rsmd.getColumnCount();
      System.out.println("\t\tgetColumnCount "+rsmd.getColumnCount());
      //System.out.println("\t\tcolumnNoNulls"+rsmd.columnNoNulls());
      //System.out.println("\t\tcolumnNullable"+rsmd.columnNullable());
      for (int i=1; i<rsmdColuumnCount+1; i++) {
      try {
        System.out.println("\t\tCatalogName "+rsmd.getCatalogName(i));
      } catch(SQLException se) {}
      try {
        System.out.println("\t\tColumnClassName "+rsmd.getColumnClassName(i));
      } catch(SQLException se) {}
      try {
        System.out.println("\t\tColumnDisplaySize "+rsmd.getColumnDisplaySize(i));
      } catch(SQLException se) {}
      try {
        System.out.println("\t\tColumnLabel "+rsmd.getColumnLabel(i));
      } catch(SQLException se) {}
      try {
        System.out.println("\t\tColumnName "+rsmd.getColumnName(i));
      } catch(SQLException se) {}
      try {
        System.out.println("\t\tColumnType "+rsmd.getColumnType(i));
      } catch(SQLException se) {}
      try {
        System.out.println("\t\tColumnTypeName "+rsmd.getColumnTypeName(i));
      } catch(SQLException se) {}
      try {
        System.out.println("\t\tPrecision "+rsmd.getPrecision(i));
      } catch(SQLException se) {}
      try {
        System.out.println("\t\tScale "+rsmd.getScale(i));
      } catch(SQLException se) {}
      try {
        System.out.println("\t\tSchemaName "+rsmd.getSchemaName(i));
      } catch(SQLException se) {}
      try {
        System.out.println("\t\tTableName "+rsmd.getTableName(i));
      } catch(SQLException se) {}
      try {
        System.out.println("\t\tisAutoIncrement "+rsmd.isAutoIncrement(i));
      } catch(SQLException se) {}
      try {
        System.out.println("\t\tisCaeSensitive "+rsmd.isCaseSensitive(i));
      } catch(SQLException se) {}
      try {
        System.out.println("\t\tisCurrency "+rsmd.isCurrency(i));
      } catch(SQLException se) {}
      try {
        System.out.println("\t\tisDefinitelyWritable "+rsmd.isDefinitelyWritable(i));
      } catch(SQLException se) {}
      try {
        System.out.println("\t\tisNullable "+rsmd.isNullable(i));
      } catch(SQLException se) {}
      try {
        System.out.println("\t\tisReadOnly "+rsmd.isReadOnly(i));
      } catch(SQLException se) {}
      try {
        System.out.println("\t\tisSearchable "+rsmd.isSearchable(i));
      } catch(SQLException se) {}
      try {
        System.out.println("\t\tisSigned "+rsmd.isSigned(i));
      } catch(SQLException se) {}
      try {
        System.out.println("\t\tisWritable "+rsmd.isWritable(i));
      } catch(SQLException se) {}
      System.out.println("\n\n");
      }
     } // ende if meta
      stmt.close();
      con.close();
    } catch(SQLException ex) {
      System.err.print("SQLException: ");
      System.err.println(ex.getMessage());
    }
  }
}

